// Copyright (c) 2013 AT&T (htpp://developer.att.com)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

#import "IAMDeltasUpdateRequest.h"
#import "IAMManager.h"
#import "JSONParser.h"
#import "IAMDeltasUpdateResponse.h"
#import "IAMMessageIndexInfo.h"

@implementation IAMDeltasUpdateRequest

-(id)init{
    self= [super init];
    
    if (!self) {
        return nil;
    }
    
    // initialize resource specific params
    self.method= @"GET";
    self.path= @"/myMessages/v2/delta";
    
    return self;
}

-(NSMutableDictionary *)prepareParameters{
    NSMutableDictionary* requestParams= [super prepareParameters];
    
    (requestParams)[@"state"]= _state? _state: @"";
    
    return requestParams;
}

/** This returns a list of deltas */
-(id)parseResponse:(id)JSONResponse forOperation:(AFHTTPRequestOperation *)operation
             error:(NSError **)error{
    
    IAMDeltasUpdateResponse* deltasUpdate= [[IAMDeltasUpdateResponse alloc] init];
    deltasUpdate.messageIndexInfo= [JSONParser getMessageIndexInfoFromJSON:(JSONResponse)[@"deltaResponse"]];
    
    // get the deltas list object from the response
    NSDictionary* deltasJSON= (JSONResponse)[@"deltaResponse"][@"delta"];
    
    if (!deltasJSON) {
        if (error) {
            
            NSString* description= [NSString stringWithFormat:
                                    @"Could not parse deltas for response: %@",
                                    deltasJSON];
            *error= [NSError errorWithDomain:@"ParsingErrorDomain"
                                        code:100
                                    userInfo:@{NSLocalizedDescriptionKey: description}];
        }
        return nil;
    }
    
    deltasUpdate.deltas= [JSONParser getDeltasFromJSON:deltasJSON];
    return deltasUpdate;
}

@end
