// Copyright (c) 2013 AT&T (htpp://developer.att.com)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

#import "IAMNotificationConnectionRequest.h"
#import "IAMQueue.h"
#import "IAMNotificationConnection.h"

#import "JSONParser.h"

@implementation IAMNotificationConnectionRequest

- (id)init
{
    self = [super init];
    if (!self) {
        return nil;
    }
    
    // default values
    _queues= @[[NSNumber numberWithInt:IAMTextQueue]];
    self.path= @"/myMessages/v2/notificationConnectionDetails";
    
    return self;
}

-(NSMutableDictionary *)prepareParameters{
    NSMutableDictionary* requestParams= [super prepareParameters];
    
    (requestParams)[@"queues"]= [self stringFromQueues:_queues];
    
    return requestParams;
}

-(NSString*)stringFromQueues:(NSArray*)queues{
    NSMutableString* queuesString= [NSMutableString string];
    
    for (int i=0; i<queues.count; ++i) {
        
        IAMQueueType type= [queues[i] intValue];
        [queuesString appendString:[self formatQueueType:type]];
        
        if (i<queues.count-1) {
            [queuesString appendString:@","];
        }
    }

    
    return queuesString;
}

-(NSString*)formatQueueType:(IAMQueueType)queue{
    
    NSString* queueType= nil;
    
    switch (queue) {
        case IAMTextQueue:
            queueType= @"TEXT";
            break;
        case IAMMMSQueue:
            queueType= @"MMS";
            break;
        default:
            queueType= @"UnknownQueueType";
            break;
    }
    
    return queueType;
}

-(id)parseResponse:(id)response forOperation:(AFHTTPRequestOperation *)operation
             error:(NSError *__autoreleasing *)error{
    
    IAMNotificationConnection*  connection= [JSONParser getConnectionFromJSON:response];
    
    if (!connection) {
        if (error) {
            // fill the error details
            NSString* errorDescription = @"Could not get Connection Details.";
            *error= [NSError errorWithDomain:@"ParsingErrorDomain"
                                        code:100
                                    userInfo:@{NSLocalizedDescriptionKey: errorDescription}];
            return nil;
        }
    }
    
    return connection;
}
@end
