// Copyright (c) 2013 AT&T (htpp://developer.att.com)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

#import "NSObject+Description.h"
#import <objc/runtime.h>

@implementation NSObject (Description)


-(id)getPropertiesDictionary{
    
    
    if ([self isKindOfClass:[NSArray class]]) {
        
        NSMutableArray* dicArray = [NSMutableArray array];
        id target= self;
        for (id item in target) {
            id itemAsDictionary= [item getPropertiesDictionary];
            [dicArray addObject:itemAsDictionary];
        }
        
        return dicArray;
        
    }
    
    // get the list of properties and save its count
    unsigned int count;
    objc_property_t* list= class_copyPropertyList( [self class], &count);
    
    // if it has properties
    if (count > 0) {
        
        // create a dictionary in which each entry is the name of the property
        // and its dictionary representation obtained by calling this method
        // recursively.
        NSMutableDictionary * propDic= [NSMutableDictionary dictionary];
        for (int i=0; i< count; ++i) {
            
            objc_property_t property = list[i];
            NSString* propertyName= [NSString stringWithUTF8String:
                                     property_getName(property)];
            
            id propertyValue= [self valueForKey:propertyName];
            
            // fill nil values with emtpy placeholders
            if (!propertyValue) {
                (propDic)[propertyName]= [NSNull null];
            }else{
                // get the dictionary representation recursively
                (propDic)[propertyName] = [propertyValue getPropertiesDictionary];
            };
        }
        
        // release memory for the list since it's not automatically managed
        free(list);
        return  propDic;
    }
    
    // if everything else fails, return the object itself
    return self;
}
@end
