// Copyright (c) 2013 AT&T (htpp://developer.att.com)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

#import "IAMManagerMock.h"
#import "IAMMessagesListRequest.h"
#import "IAMUtils.h"
#import "JSONParser.h"
#import "IAMMessageRequest.h"
#import "IAMDeltasUpdateRequest.h"
#import "IAMMessageIndexInfoRequest.h"

@implementation IAMManagerMock

-(void)sendAsynchronous:(ATTRequest *)request
                success:(void (^)(id))success
                failure:(void (^)(NSError *))failure{

    id nativeObject= nil;
    NSError* error= nil;

    NSDictionary* json= nil;
    
    // Select the appropiate file to read from, depending on the
    // request type.
    if ([request isKindOfClass:[IAMMessagesListRequest class]]) {
        
        json= [IAMUtils getJSONFromResource:@"messages-list"];
        
    }else if([request isKindOfClass:[IAMMessageRequest class]]){

        json= [IAMUtils getJSONFromResource:@"message"];

    }else if([request isKindOfClass:[IAMDeltasUpdateRequest class]]){
        
        json= [IAMUtils getJSONFromResource:@"deltas"];
    
    }else if([request isKindOfClass:[IAMMessageIndexInfoRequest class]]){
        
        json= [IAMUtils getJSONFromResource:@"message-index-info"];
        
    }else{
        NSString* description= [NSString stringWithFormat:
                                @"Don't know where to get the JSON for request of type %@",
                                NSStringFromClass([request class])];
        
        error= [NSError errorWithDomain:@"UnrecognizedTypeDomain"
                                   code:100
                               userInfo:@{NSLocalizedDescriptionKey: description}];
        failure(error);
        return;
    }
    
    // parse the actual JSON string to generate it's native representation
    nativeObject= [request parseResponse:json forOperation:nil error:&error];
    
    // if there were errors during parsing
    if (error) {
        failure(error);
        return;
    }
    
    // there were no errors, apply the success block to the resulting
    // native objects parsed from the JSON response
    success(nativeObject);
    
}

@end
