// Copyright (c) 2013 AT&T (htpp://developer.att.com)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

#import "MyCustomXCTestCase.h"
#import "OAuthToken.h"

@interface TestOAuthToken : MyCustomXCTestCase

@end

@implementation TestOAuthToken{
    OAuthToken* token;
}

- (void)setUp
{
    [super setUp];
    
    // this token expires right now.
    token= [OAuthToken oauthTokenWith:@"access_token" expirationDate:[NSDate date]
                         refreshToken:@"refresh_token"];
}

- (void)tearDown
{
    // Put teardown code here. This method is called after the invocation of each test method in the class.
    [super tearDown];
}

- (void)testIsExpired
{
    // Testing token with an expired date
    NSDate* expired=  [NSDate dateWithTimeIntervalSinceNow:-100];
    token.expirationDate= expired;
    XCTAssertTrue([token isExpired], @"Date: %@ has NOT passed.", [expired description]);
    NSLog(@"Token date:%@ is older than today: %@", token.expirationDate, [NSDate date]);

    // Testing token with a future date (not expired)
    NSDate* futureDate= [NSDate dateWithTimeIntervalSinceNow:1000000];
    token.expirationDate= futureDate;
    XCTAssertFalse([token isExpired], @"Date: %@ has passed.", [futureDate description]);
    NSLog(@"Token date:%@ is later than today: %@", token.expirationDate, [NSDate date]);
}

@end
