# AT&T In-App Messaging (IAM) SDK for iOS


With the AT&T In-App Messaging API, your users can send & receive text
& media directly through your app to any mobile number in the U.S. 

The In-App Messaging SDK  is an iOS library for natively consuming the RESTful resources for [AT&T's In-App Messaging API](http://developer.att.com/apis/in-app-messaging). The SDK abstracts away all the networking tasks so that you can focus on what matters, getting & sending your messages. 

- [Quick start](#quick-start)
	- [Requirements](#requirements)
	- [Run the Examples](#run-the-examples)
- [Overview](#overview)
	- [Request wrappers](#request-wrappers)
	- [Request management](#request-management)
	- [High level sequence diagram](#high-level-sequence-diagram)
- [Getting started with In-App Messaging SDK for iOS](#getting-started-with-in-app-messaging-sdk-for-ios)
	- [1. Getting the sources](#1-getting-the-sources)
	- [2. Install the dependencies](#2-install-the-dependencies)
		- [AFNetworking](#afnetworking)
		- [Base64](#base64)
			- [Initialize Git submodules](#initialize-git-submodules)
			- [Without Git](#without-git)
	- [3. Building the targets](#3-building-the-targets)
	- [4. Running the Unit Tests and the OAuth example](#4-running-the-unit-tests-and-the-oauth-example)
- [Using the In-App Messaging SDK in your App](#using-the-in-app-messaging-sdk-in-your-app)
	- [Usage](#usage)
		- [Create an In-App Messaging client](#create-an-in-app-messaging-client)
		- [Send an SMS](#send-an-sms)
		- [Send an MMS](#send-an-mms)
		- [More](#more)

# Quick start

Sending a message can be as easy as:

```objc
	// Initialize your request manager
	IAMManager* messagingManager= …
	
	// Send an SMS message
    [messagingManager sendSMS:@"This is a test message"
                           to:@"tel:1234567890,someone@somewhere.com"
                      failure:^(NSError* error){
                          NSLog(@"%@", error.localizedDescription);
                      }];
```

[See more usage examples](#usage)

## Requirements

- [Git](http://git-scm.com/)
- [CocoaPods](http://cocoapods.org/)

*NOTE: [GitHub SSH keys](https://help.github.com/articles/generating-ssh-keys) must be propertly setup for succesfully cloning the repo.*

## Run the Examples

1. Clone this repo : `git clone --recursive git@github.com:attdevsupport/att-sdk-ios.git`
2. Change to the source directory: `cd att-sdk-ios`
3. Initialize the Git submodules: `git submodule update --init`
4. Install cocoapods if you don't have it: `sudo gem install cocoapods`
5. Do `pod install` from within the `InAppMessaging` directory
6. Open the `InAppMessaging.xcworkspace` 
7. Build & run the sample application target `InAppMessagingSampleApp`

For more detailed information on how to build got keep go to [Getting started with In-App Messaging SDK for iOS](#getting-started-with-in-app-messaging-sdk-for-ios)


# Overview

There are two main components to the library:

 - A set of request wrappers for the HTTP requests to access the API's resources.
 - A request manager --HTTP client-- to handle all your In-App Messaging requests.

## Request wrappers

The In-App Messaging SDK provides Objective-C wrappers for the actual [REST resources](http://developer.att.com/apis/in-app-messaging/docs#resources) you want to access in your application and exposes the main request parameters.

| NSObject subtype | 
| -------- | 
| `IAMCreateMessageIndexRequest` |
| `IAMMessageRequest` |
| `IAMMessagesListRequest` |
| `IAMSendMessageRequest` |
| `IAMDeleteMessageRequest` |
| `IAMDeleteMessagesRequest` |
| `IAMDeltasUpdateRequest` |
| `IAMMessageContentRequest` |
| `IAMMessageIndexInfoRequest` |
| `IAMNotificationConnectionRequest` |
| `IAMUpdateMessageRequest` |
| `IAMUpdateMessagesRequest` |


## Request management

AT&T's In-App Messaging SDK abstracts away the networking layer by providing a request manger: `IAMManager`, think of it as an HTTP client, it creates the actual HTTP requests using an instance of `ATTRequest` or any of it's sub-classes (see previous section) and allows the developer to define the success and failure callbacks. This way you can access the API resources with a simple call:

```objc    
    // Make the request
    [iamManager sendAsynchronous:iamRequest success:^(id requestResponse) {
        // your code for success 
    } failure:^(NSError *error) {
        // your code for failure
    }];
```

## High level sequence diagram

![image](docs/iam-hl-seq-diag.png)


# Getting started with In-App Messaging SDK for iOS


## 1. Getting the sources

You can get the sources by cloning this repo with Git:

    $ git clone --recursive https://github.com/attdevsupport/att-sdk-ios
    
Optionally you can download this repo as a [Zip file](https://github.com/attdevsupport/att-sdk-ios/archive/master.zip).

    
## 2. Install the dependencies

### AFNetworking

The In-App Messaging SDK uses the wonderful library [AFNetworking](https://github.com/AFNetworking/AFNetworking), it is provided as a CocoaPod dependency. The setup for this dependency is in this [Podfile](Podfile) 

[CocoaPods](http://beta.cocoapods.org/?q=) is a dependency management system for Cocoa. If you don't have CocoaPods installed, you should. You can install it by running the commands in the OS X Terminal: 

    $ sudo gem install cocoapods
    $ pod setup

Once you have setup CocoaPods, you can install the AFNetworking Pod running the following command from the `InAppMessaging` directory:

    $ pod install
    
Open the generated XCode Workspace: `InAppMessaging.xcworkspace`. You can either double click the workspace file or use the Terminal:

    $ open InAppMessaging.xcworkspace
    
### Base64

We also need the useful [Base64](https://github.com/nicklockwood/Base64) library which is added as a Git submodule. 

#### Initialize Git submodules

If you didn't clone recursively (with the `--recursive` option), then initialize the submodule by running:

    $ git submodule update --init
    
**Note: Make sure to run this command from the root level of this repo**

#### Without Git

If you downloaded the repo as a Zip file, just download the [Base64 sources](https://github.com/nicklockwood/Base64) and put the contents under `InAppMessaging/Base64`
    
## 3. Building the targets 


To verify everything is in place, try building the library & the example in XCode. 

There are four main targets:

- `InAppMessagingSampleAp` - This is the Sample application showing how you can use the library.
- `InAppMessaging` - This is the SDK's source.
- `OAuthExample` - A very simple example of how to use the library with OAuth user consent.
- `InAppMessagingTests` - Unit Tests.

## 4. Running the Unit Tests and the OAuth example


To run the unit tests just select the `InAppMessagingTests` target in the scheme selector and click-hold the run button in XCode, then click the Test option.

You should see the execution results in the Test Navigator or in the Log Navigator.

![image](docs/TestNavigator.png)



You can also run the target `OAuthExample` which illustrates how to use the `ATTOAuthViewController` which provides a generic component to carry out the OAuth Consent Flow by presenting the authorization page to the user so he can provision the application & device to use the In-App Messaging API.

![image](docs/authorization_page.png)

# Using the In-App Messaging SDK in your App

### Using the binary

1. Add the files under `InAppMessaging` to your project
2. Link with the static library `libInAppMessaging-.a` in your *Build Phases*

### Using the sources

1. Open the XCode workspace `InAppMessaging.xcworkspace`
2. Add a new project to the workspace.
3. Link your target with the `libInAppMessaging.a` library in your target's *Build Phases*. You will also need to link with the `SystemConfiguration.framework` and `MobileCoreServices.framework`
4. Add the `-ObjC` flag in your **Other Linker Flags** section in the *Build Settings* of your target.
5. Make the `InAppMessaging` headers available to your project by adding them to your target's **Header search paths** in the *Build Settings*. 

You're now ready to start creating your own Apps with In-App Messaging support. Hack away!

## Usage

### Create an In-App Messaging client

```objc
    // initialize the app configuration
    ATTAppConfiguration* _appConfig= [[ATTAppConfiguration alloc] init];
    _appConfig.key= AppKey;
    _appConfig.secret= AppSecret;
    _appConfig.scope= AppScope;
    _appConfig.baseURL= [NSURL URLWithString:APIBaseURL];
    _appConfig.redirectURL= [NSURL URLWithString:RedirectURL];
    
    // initialize a client for In-App Messaging
    IAMManager* messagingManager= [[IAMManager alloc] initWithConfig:_appConfig andDelegate:nil];
```

### Send an SMS

```objc
	// Send an SMS message
    [messagingManager sendSMS:@"This is a test message"
                           to:@"tel:1234567890,someone@somewhere.com"
                      failure:^(NSError* error){
                          NSLog(@"%@", error.localizedDescription);
                      }];
```

### Send an MMS

```objc
	IAMSendMessageRequest* sendRequest= [[IAMSendMessageRequest alloc] init];
    sendRequest.addresses= @[@"tel:1231231230", @"someone@somewhere.com"];
    sendRequest.text= @"Hellou!";
    sendRequest.subject= @"Hi, this is an MMS message";
    
    // ADD an image to the message
    [sendRequest addImage:[UIImage imageNamed:@"image"]];
    
    [messagingManager sendAsynchronous:sendRequest success:^(id messageId) {
        NSLog(@"Succesfully sent message with Id: %@", messageId);
    } failure:^(NSError *error) {
        NSLog(@"%@", error.localizedDescription);
    }];
```

### More
Browse the [Unit Tests for `IAMManager`](InAppMessagingTests/TestMessagingManager.m) for more examples on how to consume each API resource using this library.
